/*
 * @(#)BattleShips.java 1.0 03/08/26
 *
 * Written by James McMahon
 */

import java.awt.*;
import java.util.*;
import java.applet.*;
import javax.swing.*;

public class AIPlayer extends Player {
	
	/**
	 * An EASY computer level - randomly fires at opponent board, places randomly
	 */
	public static int EASY   = 0;
	
	/**
	 * A MEDIUM computer level - fires randomly with good follow up after a hit, places randomly
	 */
	public static int MEDIUM = 1;
	
	/**
	 * A HARD computer level - fires in pattern with good follow up after a hit, places sensibly when board size allows
	 */
	public static int HARD   = 2;
	
	/**
	 * The main parent class required to set up the Board object
	 */
	private BattleShips parent = null;
	
	/**
	 * The number of squares across for the board
	 */
	private int across = 6;
	
	/**
	 * The number of squares down for the board
	 */
	private int down = 6;
	
	/**
	 * The AI difficulty level (Default is always EASY)
	 */
	private int difficulty = this.EASY;

	/**
	 * Random generator for placing ships
	 */
	private Random randNum = new Random();	

	/**
	 * Stores whether or not the last shot hit a ship
	 */
	private boolean lastShotHit = false;
	
	/**
	 * Stores whether or not a previous shot hit a new ship that hasn't been sunk yet
	 */
	private boolean newShipStruck = false;

	/**
	 * Stores the location that was last shot at and hit (when centralHit first gets set)
	 */
	private BoxLocation lastHitShotAt = null;	
	
	/**
	 * Stores the location that a ship has first been hit at
	 */
	private BoxLocation firstHitShotAt = null;
	
	/**
	 * Stores whether or not the last shot sunk a ship
	 */
	private boolean lastShotSunkShip = false;

	/**
	 * Stores the location that was last shot at (modifies medium and hard targeting)
	 */
	private BoxLocation lastShotAt = null;

	/**
	 * The actual board object
	 */
	private Board board = null;
	
	/**
	 * Creates an AIPlayer with a board of the specified sizes
	 * 
	 * @param difficultyLevel The level the computer will play at (must be valid or defaults to easy)
	 * @param boardSizeAcross The number of squares across for the board
	 * @param boardSizeDown The number of squares down for the board
	 * @param parentOwner The main applet clas necessary for creating the board object
	 */
	public AIPlayer(int difficultyLevel, int boardSizeAcross, int boardSizeDown, BattleShips parentOwner) { 
		
		if ((difficultyLevel >= this.EASY) || 
		    (difficultyLevel <= this.HARD)) {
			difficulty = difficultyLevel;
		}
		else {//Default to EASY
			difficulty = this.EASY;
		}
		
		across     = boardSizeAcross;
		down       = boardSizeDown; 
		parent     = parentOwner;

		board = new Board(across,down,false,parent);
		
		setUpShips();
	}
	
	/**
	 * Sets up the ships in a difficulty level dependent way
	 */
	private void setUpShips() {
		if ((difficulty==this.EASY) || (difficulty==this.MEDIUM)) {
			setUpShipsRandomly();
		}
		else if (difficulty==this.HARD) {
			setUpShipsSpacedOut();
		}
	}
	
	/**
	 * Sets up the ships in a completely random manner
	 */
	private void setUpShipsRandomly() {
		//Get a random int between 0,1,2 - (size-1)
 		int locationAcross = randNum.nextInt(this.across);
 		int locationDown   = randNum.nextInt(this.down);
 		
 		board.placeRandomShip(Board.ship_battleship);
 		board.placeRandomShip(Board.ship_cruiser);
 		board.placeRandomShip(Board.ship_destroyer1);
 		board.placeRandomShip(Board.ship_destroyer2);
 		board.placeRandomShip(Board.ship_mtb1);
 		board.placeRandomShip(Board.ship_mtb2);
 		board.placeRandomShip(Board.ship_mtb3);
 	}
	
	/**
	 * Sets up the ships in a random mannner but tries to minimise their number of 
	 * connections between ships if board size allows
	 */
	private void setUpShipsSpacedOut() {
 		board.placeSpacedRandomShip(Board.ship_battleship);
 		board.placeSpacedRandomShip(Board.ship_cruiser);
 		board.placeSpacedRandomShip(Board.ship_destroyer1);
 		board.placeSpacedRandomShip(Board.ship_destroyer2);
 		board.placeSpacedRandomShip(Board.ship_mtb1);
 		board.placeSpacedRandomShip(Board.ship_mtb2);
 		board.placeSpacedRandomShip(Board.ship_mtb3);
	}
	
	/**
	 * Returns the computers shot at the human players board.
	 * It requires access to the human players board box locations only in order to ensure
	 * that shots returned are in valid locations (i.e. not already fired/hit ship boxes)
	 * 
	 * It does not cheat by accessing the ship locations (Well not yet anyway ...)
	 * 
	 * @param targetableBoxesList A list of all the boxes that can be shot at (with blank or ship contents)
	 */
	public BoxLocation getShot(BoxLocationsList targetableBoxesList) {
		
		ArrayList targetBoxes = targetableBoxesList.getListOfTargetableBoxes();
		
		//a random box location to select from the full list
		int loc = randNum.nextInt(targetBoxes.size());
		
		if (difficulty==this.EASY) {//fire completely randomly from the available boxes
			BoxLocation location = (BoxLocation)targetBoxes.get(loc);
			this.lastShotAt = location;//bit redundant doesn't use it really
			return location;
		}
		else if ((lastShotHit && !lastShotSunkShip) || ((firstHitShotAt!=null) && !lastShotSunkShip)) {
			//do the good followup part for hard/medium play
			//has effect of targeting locations near the last hit spot
			//of a ship that has not been sunk yet
			
			ArrayList cutDown = null;
			
			if (lastShotHit){//the last shot hit a ship so target surrounding area
				cutDown = targetableBoxesList.getSurroundingBoxes(lastShotAt);
			}
			else {//the last shot didn't hit a ship but we hit one before and can use that location for targeting
				cutDown = targetableBoxesList.getSurroundingBoxes(lastHitShotAt);	
			}
			
			if (cutDown.isEmpty()) {//first go back to the location of the ship 
			                        //that was first hit and proceed from there
				cutDown = targetableBoxesList.getSurroundingBoxes(firstHitShotAt);	
			}
			
			//could still actually be empty hence need to catch the special cases
			if (cutDown.isEmpty()) {
				//then we can't really find the next best location so just choose a random box
				BoxLocation location = (BoxLocation)targetBoxes.get(loc);
				this.lastShotAt = location;
				return location;
			}
			else {//then can get a location near to the last hit ship spot that is our best option
				int newLoc = randNum.nextInt(cutDown.size());
				BoxLocation location = (BoxLocation)cutDown.get(newLoc);
				this.lastShotAt = location;
				return location;
			}
		}
		else {//the last shot didn't hit a ship but we must be medium or hard based so either
			if (difficulty==this.MEDIUM) {//simply fire randomly 
				BoxLocation location = (BoxLocation)targetBoxes.get(loc);
				this.lastShotAt = location;
				return location;
			}
			else if (difficulty==this.HARD) {//fire in diagonal pattern
				if (lastShotAt == null) {//simply choose any random starting place
					BoxLocation location = (BoxLocation)targetBoxes.get(loc);
					this.lastShotAt = location;
					return location;
				}
				else {//want to start pattern matching diagonally from the last shot
					ArrayList diagonals = targetableBoxesList.getSurroundingDiagonalBoxes(lastShotAt);
					//might be an empty array list if no valid diagonals
				
					if (diagonals.isEmpty()) {//choose new random start
						BoxLocation location = (BoxLocation)targetBoxes.get(loc);
						this.lastShotAt = location;
						return location;
					}
					else {//choose random diagonal	
						int newLoc = randNum.nextInt(diagonals.size());
						BoxLocation location = (BoxLocation)diagonals.get(newLoc);
						this.lastShotAt = location;
						return location;
					}
				}
			}
		}
		return null;//should never reach here but what the hey
	}
	
	/**
	 * Needs to be called after every shot that the ai makes. Helps theharder AI levels target next shot.
	 *
	 * @param hitStatus Whether the last shot hit a ship
	 * @param sunk Whether the last shot sunk a ship or not
	 */
	public void setLastShotDetails(boolean hitStatus, boolean sunk) {
		this.lastShotHit = hitStatus;
		this.lastShotSunkShip = sunk;
		
		if (firstHitShotAt==null && lastShotHit) {//must have hit a new ship
			newShipStruck = true;
			firstHitShotAt = this.lastShotAt;//store the first hit location
		}
		
		if (lastShotHit && !lastShotSunkShip) {//update the last shot hit at
			lastHitShotAt = this.lastShotAt;
		}
		else if (lastShotSunkShip) {//the ship was sunk so don't want the central hit shot stored
			newShipStruck = false;
			lastHitShotAt = this.lastShotAt;//lets us follow the pattern from here if necessary
			firstHitShotAt = null;
		}
	}
	
	/**
	 * Returns a board set to this players size settings
	 * 
	 * @return The board object of this player
	 */
	public Board getPlayersBoard() {
		return board;
	}
}