/*
 * @(#)BattleShips.java 1.0 03/08/26
 *
 * Written by James McMahon
 */

import java.awt.*;
import java.applet.*;
import javax.swing.*;

public class BattleShips extends JApplet {
	
	/**
	 * The menu bar of the applet
	 */
	private MenuBarPanel menuBar = new MenuBarPanel(this);
	
	/**
	 * The board that is used for the player
	 */
	private Board playerBoard = null;
	
	/**
	 * The board that is used for the computer player
	 */
	private Board computerBoard = null;
	
	/**
	 * Sets up an AI player
	 */
	private AIPlayer aiPlayer = null;
	
	/**
	 * Sets up a human player
	 */
	private HumanPlayer humanPlayer = null;
	
	/**
	 * The mouse listener for the player's board
	 */
	private BoardMouseListener playerMouseListener = null; 
	
	/**
	 * The mouse listener for the computer's board
	 */
	private BoardMouseListener computerMouseListener = null; 
	
	/**
	 * Stores the users options for the current board size wanted for playing on (boxes across)
	 * Default is 6 (for a 6x6 board)
	 */
	private int boardsBoxesAcross = 6;
	
	/**
	 * Stores the users options for the current board size wanted for playing on (boxes down)
	 * Default is 6 (for a 6x6 board)
	 */
	private int boardsBoxesDown   = 6;
	
	/**
	 * The computer player's skill level (Defaulted to EASY)
	 */
	private int computerPlayerSkill = AIPlayer.EASY;
	
	/**
	 * Starts the main game going
	 */
	public void init() {
		//set up the menubar
		this.setSize(601,300+menuBar.getPreferredSize().height);
		this.setJMenuBar(menuBar);
		
		//start a new game
		startNewGame();
		
		this.validate();
		this.repaint();
	}

	/**
	 * Using the globally set parameters for board size et al it sets up and starts a new game
	 */
	public void startNewGame() {
		//ensure that the menu clear ships option is available 
		menuBar.setClearShipEnabled(true);

		Container content = this.getContentPane();
		content.setLayout(new BorderLayout());		

		content.removeAll();
		if (playerBoard != null) {
			content.remove(playerBoard);
			content.remove(computerBoard);
		}

		aiPlayer = new AIPlayer(this.computerPlayerSkill, this.boardsBoxesAcross, this.boardsBoxesDown, this);
		humanPlayer = new HumanPlayer(this.boardsBoxesAcross, this.boardsBoxesDown, this);

		playerBoard   = humanPlayer.getPlayersBoard();
		computerBoard = aiPlayer.getPlayersBoard();

		playerMouseListener   = new BoardMouseListener(playerBoard, false);
	 	computerMouseListener = new BoardMouseListener(computerBoard, true);

		playerBoard.addMouseListener(playerMouseListener);
		computerBoard.addMouseListener(computerMouseListener);		

		//because user first has to place ships make sure they can't cheat and start firing first
		computerMouseListener.turnOffListener();

		content.add(playerBoard, BorderLayout.WEST);
		content.add(computerBoard, BorderLayout.EAST);

		this.validate();
		this.repaint();

	    JOptionPane.showMessageDialog(this, 
					 		  		  new JLabel("Place your BattleShip (5 locations)"),
					 		  		  "Placing Ships",
					          		  JOptionPane.INFORMATION_MESSAGE);
	}

	/**
	 * Resets the player board's current ship that is being placed. 
	 * (Has no effect if no ships in process of being placed) 
	 */
	public void resetCurrentShipPlacement() {
		//only ever need to reset the human players board's current ship
		playerBoard.resetCurrentShipPlacement();
	}

	/**
	 * Changes the current player from human to computer to human.
	 * The computer makes it's move before changed back.
	 */
	public void makeComputerPlayerMove() {
		
		//first simply turn off the computerboard mouse listener
		computerMouseListener.turnOffListener();
		
		if (!computerBoard.allShipsSunk()) {
		
			//Get the ai's shot and fire it at the board
			BoxLocation locShot = aiPlayer.getShot(playerBoard.getValidShotLocations());	
			playerBoard.dealWithManualShot(locShot.getReference().x, locShot.getReference().y);
		
			//make sure the AI knows what it's shot did to the board
			aiPlayer.setLastShotDetails(playerBoard.lastShotHitShip(), playerBoard.lastShotSunkShip());
		
			if (!playerBoard.allShipsSunk()) {
				//the human can go again so start listening for clicks again
				computerMouseListener.turnOnListener();			
			}
			else {//game is over the human lost
				JOptionPane.showMessageDialog(this, 
				 		  		  new JLabel("Why not try something a little less taxing next time!"),
				 		  		  "Bad Luck! You LOST!!! Ha Ha Ha ...",
				          		  JOptionPane.INFORMATION_MESSAGE);
			}
		}
		else { //Argh The computer lost - human won
	    	JOptionPane.showMessageDialog(this, 
				 		  		  new JLabel("You won this time. What about the next time?"),
				 		  		  "Congratulations and Celebrations ...",
				          		  JOptionPane.INFORMATION_MESSAGE);
		}
	}

	/**
	 * Disables the clearShip menu option (to stop the user cheating and 
	 * clearing the last placed ship when they should be firing!)
	 */
	public void allShipsPlaced() {
		menuBar.setClearShipEnabled(false);
		playerMouseListener.turnOffListener();
		computerMouseListener.turnOnListener();
	}

	/**
	 * Sets up the games options ready for the next game to be started
	 */
	public void setOptions(int boardBoxesAcross, int boardBoxesDown, int computerPlayerLevel) {
		this.boardsBoxesAcross   = boardBoxesAcross;
		this.boardsBoxesDown     = boardBoxesDown;
		this.computerPlayerSkill = computerPlayerLevel;
	}

	//Don't have any explicit threads so no need to do anything in here
	public void start() {
		super.start();
	}

	public void stop() {
		super.stop();	
	}
}