import javax.swing.*;
import java.util.*;
import java.awt.Point;

/*
 * @(#)BattleShips.java 1.0 03/08/26
 *
 * Written by James McMahon
 */
 
public class BoxLocationsList {
	
	private ArrayList list = null;
	
	/** 
	 * Create an empty BoxLocationsList
	 */
	public BoxLocationsList() {
		list = new ArrayList();
	}
	
	/** 
	 * Create a BoxLocationsList with the passed in contents
	 */
	public BoxLocationsList(ArrayList listOfBoxLocations) {
		list = new ArrayList();
		list = listOfBoxLocations;
	}
	
	/**
	 * Adds the box to the list of all boxes
	 * 
	 * @param box The box location to be added
	 */
	public void addBoxLocationToList(BoxLocation box) {
		list.add(box);
	}
	
	/**
	 * Gets an arraylist representation of all available box locations
	 * @return The array list containing all of the box locations
	 */
	public ArrayList getListOfAllBoxes() {
		return list;
	}
	
	/**
	 * Gets an arraylist representation of all available box locations that can still be fired at.
	 * (i.e. Blank or Ship Locations - not hit or missed shot ones)
	 * @return The array list containing all of the box locations that can be fired at
	 */
	public ArrayList getListOfTargetableBoxes() {
		ArrayList temp = new ArrayList();
		
		ListIterator i = list.listIterator();

		//only add blank tiles or tiles with ships (contents above 10)	
		while (i.hasNext()) {
			BoxLocation tempBox = (BoxLocation)i.next();
			if ((tempBox.getContents() == Board.blank) || 
			    (tempBox.getContents() >= 10)) {
				temp.add(tempBox);
			}
       	}
		return temp;
	}
	
	/**
	 * Gets an arraylist representation of all available box locations that are blank only.
	 * (i.e. not hit or missed shot ones or ship locations)
	 * @return The array list containing all of the box locations that can be fired at
	 */
	public ArrayList getListOfBlankBoxes() {
		ArrayList temp = new ArrayList();
		
		ListIterator i = list.listIterator();

		//only add blank tiles 
		while (i.hasNext()) {
			BoxLocation tempBox = (BoxLocation)i.next();
			if (tempBox.getContents() == Board.blank) {
				temp.add(tempBox);
			}
       	}
		return temp;
	}
	
	/**
     * Returns a list of all valid shot location boxes that surround box loc 
	 * in a horizontal/vertical direction. It may be empty however.
	 */
	public ArrayList getSurroundingBoxes(BoxLocation loc) {
		ArrayList temp = new ArrayList();
		
		ListIterator i = this.getListOfAllBoxes().listIterator();
		
		Point center = loc.getReference();
		
		//the four box locationsthat we are willing to accept in terms of reference locations
		Point top    = new Point(center.x, center.y-1);
		Point right  = new Point(center.x+1, center.y);
		Point left   = new Point(center.x-1, center.y);
		Point bottom = new Point(center.x, center.y+1);
		
		while (i.hasNext()) {
			BoxLocation tempLoc = (BoxLocation)i.next();
			if ((tempLoc.getReference().equals(top))   ||
				(tempLoc.getReference().equals(right)) ||
				(tempLoc.getReference().equals(left))  ||
				(tempLoc.getReference().equals(bottom))) {
				temp.add(tempLoc);
			}
       	}
       	return temp;
	}

	/**
     * Returns a list of all blank location boxes that surround box loc 
	 * in a horizontal/vertical direction. It may be empty however.
	 */
	public ArrayList getSurroundingBlankBoxes(BoxLocation loc) {
		ArrayList temp = new ArrayList();
		
		ListIterator i = this.getListOfAllBoxes().listIterator();
		
		Point center = loc.getReference();
		
		//the four box locationsthat we are willing to accept in terms of reference locations
		Point top    = new Point(center.x, center.y-1);
		Point right  = new Point(center.x+1, center.y);
		Point left   = new Point(center.x-1, center.y);
		Point bottom = new Point(center.x, center.y+1);
		
		while (i.hasNext()) {
			BoxLocation tempLoc = (BoxLocation)i.next();
			if ((tempLoc.getReference().equals(top))   ||
				(tempLoc.getReference().equals(right)) ||
				(tempLoc.getReference().equals(left))  ||
				(tempLoc.getReference().equals(bottom))) {
					if (tempLoc.getContents() == Board.blank) {//only add it if it is blank
						temp.add(tempLoc);
					}
			}
       	}
       	return temp;
	}

	/**
	 * Returns a list of all valid shot location boxes that surround box loc 
	 * in a diagonal direction. It may be empty however.
	 */
	public ArrayList getSurroundingDiagonalBoxes(BoxLocation loc) {
		ArrayList temp = new ArrayList();
		
		ListIterator i = this.getListOfAllBoxes().listIterator();
		
		Point center = loc.getReference();
		
		//the four box locationsthat we are willing to accept in terms of reference locations
		Point topLeft     = new Point(center.x-1, center.y-1);
		Point topRight    = new Point(center.x+1, center.y-1);
		Point bottomLeft  = new Point(center.x-1, center.y+1);
		Point bottomRight = new Point(center.x+1, center.y+1);
		
		while (i.hasNext()) {
			BoxLocation tempLoc = (BoxLocation)i.next();
			if ((tempLoc.getReference().equals(topLeft))    ||
				(tempLoc.getReference().equals(topRight))   ||
				(tempLoc.getReference().equals(bottomLeft)) ||
				(tempLoc.getReference().equals(bottomRight))) {
				temp.add(tempLoc);
			}
       	}
       	return temp;
	}

	/**
	 * Checks to see if point target is contained within the sphere of any of the 
	 * boxes in the list's boundaries.
	 * <p>
	 * If the point is in one of the boxes then it returns that box otherwise it returns null
	 * 
	 * @param target The point that is to be checked
	 * @return a BoxLocation that surrounds the target or null if one doesn't exist
	 */
	public BoxLocation getBoxLocationContainingPoint(Point target) {

		ListIterator i = list.listIterator();
		
		while (i.hasNext()) {
			BoxLocation temp = (BoxLocation)i.next();
			if (temp.containsPoint(target)) {
				return temp;
			}
       	}
		return null;
	}
	
	/**
	 * Checks to see if point reference is available in the list.
	 * <p>
	 * If the point refers to one of the boxes then it returns that box otherwise 
	 * it returns null
	 * 
	 * @param reference The reference of the box that is to be found
	 * @return a BoxLocation that has that reference or null if one doesn't exist
	 */
	public BoxLocation getBoxLocation(Point reference) {

		ListIterator i = list.listIterator();
		
		while (i.hasNext()) {
			BoxLocation temp = (BoxLocation)i.next();

			if (temp.getReference().equals(reference)) {
				return temp;
			}
       	}
		return null;
	}
}