/*
 * @(#)BattleShips.java 1.0 03/08/26
 *
 * Written by James McMahon
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import javax.swing.*;

public class OptionsPanel extends JPanel {
	
	private JRadioButton easyArt     = new JRadioButton("Easy Art Degree", true);
	private JRadioButton mediumMedia = new JRadioButton("Medium Media Studies Degree", false);
	private JRadioButton compSciHard = new JRadioButton("Hard Science Degree", false);
	
	private JRadioButton smallBoard  = new JRadioButton("Small Board (6x6)", true);
	private JRadioButton mediumBoard = new JRadioButton("Medium Board (10x10)", false);
	private JRadioButton largeBoard  = new JRadioButton("Large Board (15x15)", false);
	private JRadioButton customBoard = new JRadioButton("Custom Board (?x?)", false);
	
	private ButtonGroup aiGroup    = new ButtonGroup();
	private ButtonGroup boardGroup = new ButtonGroup();
	
	private JLabel aiDifficulty = new JLabel("<HTML><h3><u>Computer Opponent Skill Level:</u></h3></HTML>");
	private JLabel boardSize    = new JLabel("<HTML><h3><u>Board Size:</u></h3></HTML>", JLabel.CENTER);

	private int customAcrossBox = 6;//default
	private int customDownBox   = 6;//default
	private boolean customSetUpStatus = false;//whether the user has set a custom board or not
	
	private final JTextField squaresAcross  = new JTextField(5);
	private final JTextField squaresDown    = new JTextField(5);
	private final JCheckBox  setBoxes       = new JCheckBox("Set Board", false);	
	
	private JLabel instructions = new JLabel("Specify a custom range of boxes (between 5 and 20)");
	private JLabel acrossLabel  = new JLabel("Across");
	private JLabel downLabel    = new JLabel("Down");		
	
	private JPanel aiSettings    		 = new JPanel(new FlowLayout());	
	private JPanel internalBoardSettings = new JPanel(new GridLayout(2,2));
	private JPanel boardSettings 	     = new JPanel(new BorderLayout());		
	private JPanel customBoardSettings   = makeCustomBoardInputPanel();	

	/**
	 * Sets up an options panel which can be used to modify some of the game options.
	 * As soon as this is opened up (in effect) it should really start off a new game, 
	 * hence warn the user before hand that a new game will automatically be begun.
	 */
	public OptionsPanel() {
		
		//set up the JPanel dimensions
		this.setPreferredSize(new Dimension(310,290));
		aiSettings.setPreferredSize(new Dimension(280,130));
		internalBoardSettings.setPreferredSize(new Dimension(310,130));
		boardSettings.setPreferredSize(new Dimension(310,140));
		
		//Set up the buttongroups
		aiGroup.add(easyArt);
		aiGroup.add(mediumMedia);
		aiGroup.add(compSciHard);
		
		boardGroup.add(smallBoard);
		boardGroup.add(mediumBoard);
		boardGroup.add(largeBoard);
		boardGroup.add(customBoard);				
		
		//set up the action commands
		smallBoard.setActionCommand("SMALL");
		mediumBoard.setActionCommand("MEDIUM");
		largeBoard.setActionCommand("LARGE");
		customBoard.setActionCommand("CUSTOM");
		easyArt.setActionCommand("EASY");
		mediumMedia.setActionCommand("MEDIUM");//this is a hack to speed up comparisons at getComputerPlayerLevel etc
		compSciHard.setActionCommand("HARD");  //so be careful if adding any more listeners for these actioncommands
		
		//set up the options for the computer player difficulty
		aiSettings.add(aiDifficulty);
		aiSettings.add(easyArt);
		aiSettings.add(mediumMedia);
		aiSettings.add(compSciHard);
		
		//set up the options for the level type (small up to large or custom)  
		boardSettings.add(boardSize, BorderLayout.NORTH);
		internalBoardSettings.add(smallBoard);
		internalBoardSettings.add(mediumBoard);
		internalBoardSettings.add(largeBoard);
		internalBoardSettings.add(customBoard);
		
		boardSettings.add(internalBoardSettings, BorderLayout.CENTER);
		
		//make sure that the custom board settings are disabled
		disableCustomBoard();
		
		boardSettings.add(customBoardSettings, BorderLayout.SOUTH);
		
		//the listener that gets added to the board size selector buttons 
		//so that the custom option is sensibly available 
		ActionListener listener = new ActionListener() { 
			public void actionPerformed(ActionEvent ae) {
		 		if (ae.getActionCommand().equals("CUSTOM")) {
			 		if (((JRadioButton)ae.getSource()).isSelected()) {
		 				JOptionPane.showMessageDialog(boardSettings, 
										 "Please enter a valid range then click on Set Board",
										 "Custom Board Configuration",
										 JOptionPane.INFORMATION_MESSAGE);
			 			enableCustomBoard();
			 		}
			 	}
		 		else {//another buton was selected so disable the custom board settings
					disableCustomBoard();
		 		}
		 	}
		 };
		 		
		customBoard.addActionListener(listener);
		smallBoard.addActionListener(listener);
		mediumBoard.addActionListener(listener);
		largeBoard.addActionListener(listener);
		
		this.add(aiSettings, BorderLayout.NORTH);
		this.add(boardSettings, BorderLayout.CENTER);
	}
	
	/**
	 * Make sure that the custom board selector has been enabled
	 */
	private void enableCustomBoard() {
		customSetUpStatus = true;
		squaresAcross.setEnabled(true);
		squaresAcross.setText("6");
 		squaresDown.setEnabled(true);
		squaresDown.setText("6");
		setBoxes.setEnabled(true);
		setBoxes.setSelected(false);
		instructions.setEnabled(true);
	    acrossLabel.setEnabled(true);
		downLabel.setEnabled(true);
	}
	
	/**
	 * Make sure that the custom board selector has been disabled 
	 */
	private void disableCustomBoard() {
		customSetUpStatus = false;
		squaresAcross.setEnabled(false);
 		squaresDown.setEnabled(false);
		setBoxes.setEnabled(false);
		instructions.setEnabled(false);
	    acrossLabel.setEnabled(false);
		downLabel.setEnabled(false);
	}

	/**
	 * returns the user entered custom settings for the number of boxes across the board.
	 * If not set then it returns the default small board size
	 */
	public int getBoxesAcrossCustomSettings() {
		if (isCustomBoardValuesSet()) {
			return customAcrossBox;
		}
		else {
			return 6;
		}
	}
	
	/**
	 * returns the user entered custom settings for the number of boxes across the board.
	 * If not set then it returns the default small board size
	 */	
	public int getBoxesDownCustomSettings() {
		if (isCustomBoardValuesSet()) {
			return customDownBox;
		}
		else {
			return 6;
		}
	}

	/**
	 * Returns whether or not the user has succesfully set up the 
	 * board ranges or not
	 */
	public boolean isCustomBoardValuesSet() {
		return setBoxes.isSelected();
	}

	/**
	 * Returns either "EASY", "MEDIUM", or "HARD" based values from the AIPlayer set
	 */
	public int getComputerPlayerLevel() {
		ButtonModel model = aiGroup.getSelection();	
		String value = model.getActionCommand();
		
		if (value.equals("EASY")) {
			return AIPlayer.EASY;
		}
		else if (value.equals("MEDIUM")) {
			return AIPlayer.MEDIUM;
		}
		else if (value.equals("HARD")) {
			return AIPlayer.HARD;
		}
		else {//should never ever happen but just in case return the default easy option
			return AIPlayer.EASY;
		}
	}
	
	/**
	 * Returns either "SMALL", "MEDIUM", "LARGE" or "CUSTOM"
	 */
	public String getBoardSizeWanted() {
		ButtonModel model = boardGroup.getSelection();	
		return model.getActionCommand();
	}
	
	/**
	 * Sets up the custom board size selector with the necessary features to force
	 * it into a set range
	 */
	private JPanel makeCustomBoardInputPanel() {
		JPanel customNumbersPanel 	 = new JPanel();
		customNumbersPanel.setPreferredSize(new Dimension(150,50));
		
		customNumbersPanel.add(instructions);
		customNumbersPanel.add(acrossLabel);
		customNumbersPanel.add(squaresAcross);
		customNumbersPanel.add(downLabel);
		customNumbersPanel.add(squaresDown);
		customNumbersPanel.add(setBoxes);
		
		setBoxes.setActionCommand("setRange");
		setBoxes.addActionListener(new ActionListener() { 
			public void actionPerformed(ActionEvent ae) {
		 		if (((JCheckBox)ae.getSource()).isSelected()) {
		 			try {
		 				int tempAcross = Integer.parseInt(squaresAcross.getText());
		 				int tempDown   = Integer.parseInt(squaresDown.getText());

		 				if (((tempAcross >= 5) && (tempAcross <= 20)) && 
		 				    ((tempDown >= 5) && (tempDown <= 20))) {

		 				customAcrossBox = tempAcross;
		 				customDownBox   = tempDown;
		 				JOptionPane.showMessageDialog(boardSettings, 
													  "The new board is accepted",
													  "Board Configured",
													  JOptionPane.INFORMATION_MESSAGE);
						disableCustomBoard();
						}
						else {
		 					throw new NumberFormatException();
		 				}		
		 			}
		 			catch (NumberFormatException nfe) {
		 				setBoxes.setSelected(false);
		 				JOptionPane.showMessageDialog(boardSettings, 
													  "Couldn't parse the ranges - Ensure they are valid integers in a valid range",
											 		  "Invalid Integer Range",
													  JOptionPane.ERROR_MESSAGE);
		 			}
		 		}
		 		else {
		 			squaresAcross.setText("6");
		 			squaresDown.setText("6");
		 			JOptionPane.showMessageDialog(boardSettings, 
												  "The range has now been reset to small",
											      "Board Reset",
												  JOptionPane.INFORMATION_MESSAGE);
		 		}
	 		}
	 	});
		return customNumbersPanel;
	}
}